'use strict';

exports.__esModule = true;
exports.default = undefined;

var _class, _temp;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _dropdown = require('./dropdown.styl');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var _default = (_temp = _class = function (_PureComponent) {
  _inherits(_default, _PureComponent);

  function _default(props) {
    _classCallCheck(this, _default);

    var _this = _possibleConstructorReturn(this, _PureComponent.call(this, props));

    _this.keydown = _this.keydown.bind(_this);
    _this.handleEnterKey = _this.handleEnterKey.bind(_this);
    _this.handleEscKey = _this.handleEscKey.bind(_this);
    _this.handleArrowKeys = _this.handleArrowKeys.bind(_this);
    _this.getSuggestions = _this.getSuggestions.bind(_this);
    _this.filterSuggestions = _this.filterSuggestions.bind(_this);
    _this.acceptSuggestion = _this.acceptSuggestion.bind(_this);
    _this.state = {
      suggestions: [],
      highlightedIdx: 0,
      selectedIdx: null
    };
    return _this;
  }

  _default.prototype.componentDidMount = function componentDidMount() {
    document.addEventListener('keydown', this.keydown, false);

    this.setState({
      suggestions: this.getSuggestions()
    });
  };

  _default.prototype.componentWillUnmount = function componentWillUnmount() {
    document.removeEventListener('keydown', this.keydown, false);
  };

  _default.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
    var value = nextProps.value;


    if (this.props.value !== value) {
      this.setState({
        suggestions: this.filterSuggestions(value),
        // reset selector to top each time value changes
        highlightedIdx: 0
      });
    }
  };

  _default.prototype.componentDidUpdate = function componentDidUpdate() {
    if (this.state.suggestions.length === 0) {
      this.props.onClose();
    }
  };

  _default.prototype.keydown = function keydown(evt) {
    switch (evt.keyCode) {
      case 9: // tab key
      case 13:
        // enter key
        this.handleEnterKey(evt);
        break;
      case 27:
        // esc key
        this.handleEscKey(evt);
        break;
      case 38: // up key
      case 40:
        // down key
        this.handleArrowKeys(evt, evt.keyCode);
        break;
    }
  };

  _default.prototype.handleEnterKey = function handleEnterKey(evt) {
    evt.preventDefault();
    this.acceptSuggestion();
  };

  _default.prototype.handleEscKey = function handleEscKey(evt) {
    evt.preventDefault();
    this.props.onClose();
  };

  _default.prototype.handleArrowKeys = function handleArrowKeys(evt, keyCode) {
    evt.preventDefault();

    var highlightedIdx = this.state.highlightedIdx;

    var isDownKey = keyCode === 40;

    // the furthest down it can go before wrapping
    var max = this.state.suggestions.length - 1;

    // determine the next position
    var newIdx = highlightedIdx !== null ? isDownKey ? highlightedIdx + 1 : highlightedIdx - 1 : isDownKey ? 0 : max;

    this.setState({
      // make sure it doesn't go out of bounds by resetting to opposite side
      highlightedIdx: isDownKey ? newIdx <= max ? newIdx : 0 : newIdx >= 0 ? newIdx : max
    });
  };

  _default.prototype.getSuggestions = function getSuggestions() {
    var attributes = this.props.attributes;
    var selectedIdx = this.state.selectedIdx;


    return selectedIdx !== null && selectedIdx > -1 ? attributes[selectedIdx].enumerations : attributes.map(function (_ref) {
      var name = _ref.name;
      return name;
    });
  };

  _default.prototype.acceptSuggestion = function acceptSuggestion() {
    var idx = this.state.highlightedIdx;
    var suggestion = this.state.suggestions[idx];
    var append = ' ';

    this.props.onSelect('' + suggestion + append);
  };

  _default.prototype.filterSuggestions = function filterSuggestions(value) {
    return this.getSuggestions().filter(function (v) {
      return new RegExp(escape(value), 'i').test(v);
    });
  };

  _default.prototype.render = function render() {
    var _this2 = this;

    var style = {
      left: (this.props.offsetX || 0) + 'px',
      top: (this.props.offsetY || 0) + 'px'
    };

    return _react2.default.createElement(
      _dropdown.Container,
      {
        tabIndex: '0',
        onFocus: this.props.onFocus,
        onBlur: this.props.onBlur,
        style: style },
      _react2.default.createElement(
        _dropdown.Suggestions,
        null,
        this.state.suggestions.map(function (suggestion, key) {
          return _react2.default.createElement(
            _dropdown.Suggestion,
            {
              key: key,
              active: _this2.state.highlightedIdx === key,
              onClick: _this2.acceptSuggestion,
              onMouseOver: function onMouseOver() {
                return _this2.setState({ highlightedIdx: key });
              } },
            suggestion
          );
        })
      ),
      _react2.default.createElement(
        _dropdown.Section,
        { center: true },
        _react2.default.createElement(
          _dropdown.Helper,
          null,
          _react2.default.createElement(
            _dropdown.KeyOutline,
            null,
            '\u25B2'
          ),
          _react2.default.createElement(
            _dropdown.KeyOutline,
            null,
            '\u25BC'
          ),
          'to navigate'
        ),
        _react2.default.createElement(
          _dropdown.Helper,
          null,
          _react2.default.createElement(
            _dropdown.KeyOutline,
            { long: true },
            '\u21B5'
          ),
          'to select'
        )
      ),
      _react2.default.createElement(
        _dropdown.Section,
        { center: true },
        _react2.default.createElement(
          _dropdown.Link,
          {
            target: '_blank',
            href: 'https://timber.io/docs/app/console/searching' },
          'Learn more'
        )
      )
    );
  };

  return _default;
}(_react.PureComponent), _class.propTypes = { // eslint-disable-line
  value: _propTypes2.default.string,
  attributes: _propTypes2.default.array,
  onSelect: _propTypes2.default.func,
  onFocus: _propTypes2.default.func,
  onBlur: _propTypes2.default.func,
  onClose: _propTypes2.default.func,
  offsetX: _propTypes2.default.number,
  offsetY: _propTypes2.default.number
}, _temp);

exports.default = _default;
module.exports = exports['default'];